#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include "LCD_Ili9341.h"
#include "LED.h"
#include "systime.h"
#include "kbd.h"

void DrawSomeShapes();
void Signal_v_sliko(float signal);
void izris_ekrana();
void reset_tocke ();
float max_vrednost (int dolzina_tabele, float *signal);
float min_vrednost (int dolzina_tabele, float *signal);
float Izracun_Upp(int dolzina_tabele, float *signal);
float Izracun_Udc(float *signal, int dolzina_tabele);
int Izracun_frek (float *signal, float Udc, uint32_t cas_zajema);
void ADC_Init();
uint16_t ADC_read();
void ADC_selectchannel(char channel);
void Refresh_display();

#define color C_WHITE // barva okvirja in pomoznih crt
#define colort C_GREEN // barva izrisane toke
#define colorl C_GREEN // barva izrisanega signala

int ii = 1;
int tockax1 = 0;
int tockax2 = 0;
int tockay1 = 140;
int tockay2 = 0;
char str[20];
float Umax = 0;
float Umin = 0;
char channel = 0; // ADC0

int main(void)
{
	LCD_Init();
	ADC_Init();
	KBD_Init();
	Systime_Init();
	izris_ekrana();
	sei();
		
	UG_FontSelect(&RFONT_8X12); // font size
	UG_SetForecolor	(C_WHITE); // font color
	UG_SetBackcolor	(C_BLACK); // font background
	
	float signal [320] = {0};
	float napetost = 0; // vrednost napetosti iz AD
	int dolzina_tabele = 320; // dolzina arraya signal[]
	int tipka = 0;
	int s_i = 0;
	int frekvenca = 0;
	uint16_t AD_vred;
	float Upp;
	float Udc;
	uint32_t cas_pred; // cas pred zajemom vrednosti ADC
	uint32_t cas_po; // cas po zajemu vrednosti ADC
	uint32_t cas_zajema; // cas zajema 320ih vrednosti ADC 

	ADC_selectchannel(channel);
	
	while (1)
	 {
		KBD_Read();
		tipka = KBD_GetKey();
		
		if(tipka != 0)
			{
				Refresh_display();
				
				cas_pred = GetSysTick();
					for(s_i=0;s_i<320;s_i++) // zajem 320ih podatkov
						{
							AD_vred = ADC_read();
							napetost = (3.3*AD_vred)/1024;
							signal[s_i]=napetost; // shrani vrednost napetosti na pravilno mesto v zbirki signal
						}
				cas_po = GetSysTick();		
				
					for(ii=0;ii<320;ii++) Signal_v_sliko(signal[ii]);
					reset_tocke ();
				
				cas_zajema = cas_po - cas_pred;
				Umax = max_vrednost(dolzina_tabele, signal);
				Umin = min_vrednost(dolzina_tabele, signal);
				Udc = Izracun_Udc(signal, dolzina_tabele);
				frekvenca = Izracun_frek(signal, Udc, cas_zajema);
				Upp = Izracun_Upp(dolzina_tabele, signal);
					if(Upp==0) frekvenca = 0;
				
					sprintf(str, "Upp: %.2fV", Upp);
					UG_PutString(5,5,str); // izpis Upp
					sprintf(str, "Umax: %.2fV", Umax);
					UG_PutString(106,5,str); // izpis Umax
					sprintf(str, "Umin: %.2fV", Umin);
					UG_PutString(106,22,str); // izpis Umin
					sprintf(str, "Udc: %.2fV", Udc);
					UG_PutString(5, 22,str); // izpis Udc
					sprintf(str, "f: %4dHz", frekvenca);
					UG_PutString(218, 5, str); // izpis Frekvence
			}
	 }
}

void Signal_v_sliko(float signal)
{
	int signalx = 0;
	int signaly = 0;
	
	signalx = ii; // izracun x koordinate tocke
	signaly = (-20*signal+140)-1; // izracun y koordinate tocke
	
	if (signaly < 39) signaly = 38; // omejitev na zgornji rob okvirja
	if (signaly > 239) signaly = 238; // omejitev na spodnji rob okvirja
	
	UG_DrawPixel(signalx, signaly, colort); //izris tocke
	
		tockax2 = signalx;
		tockay2 = signaly;
	
	UG_DrawLine(tockax1,tockay1,tockax2,tockay2,colorl);  // povezava tock signala
	
		tockax1 = tockax2;
		tockay1 = tockay2;
}

void izris_ekrana()
{
	UG_DrawFrame(0,38,319,239,color);  // izris okvirja
	UG_DrawLine(160,38,160,239,color); // izris navpicne crte
	UG_DrawLine(0,140,319,140,color); // izris vodoravne crte

	UG_DrawLine(32,137,32,143,color); // izris pomoznih crt na levi strani (x-os)
	UG_DrawLine(64,137,64,143,color);
	UG_DrawLine(96,137,96,143,color);
	UG_DrawLine(128,137,128,143,color);
	UG_DrawLine(160,137,160,143,color);

	UG_DrawLine(192,137,192,143,color); // izris pomoznih crt na desni strani (x-os)
	UG_DrawLine(224,137,224,143,color);
	UG_DrawLine(256,137,256,143,color);
	UG_DrawLine(288,137,288,143,color);

	UG_DrawLine(157,60,163,60,color); // izris pomoznih crt na zgornji strani (y-os)
	UG_DrawLine(157,80,163,80,color);
	UG_DrawLine(157,100,163,100,color);
	UG_DrawLine(157,120,163,120,color);

	UG_DrawLine(157,160,163,160,color); // izris pomoznih crt na spodnji strani (y-os)
	UG_DrawLine(157,180,163,180,color);
	UG_DrawLine(157,200,163,200,color);
	UG_DrawLine(157,220,163,220,color);
	
	UG_PutString(5,226,"Y: 1V/div"); // izpis Y 
	UG_PutString(199,226,"X: 4.8ms/div"); // izpis X
}

float max_vrednost (int dolzina_tabele, float *signal)
{
	int n = 0;
	float max_val = signal[0];
	for(n=0;n<dolzina_tabele;n++)
	 {
		if (signal[n]>max_val) max_val=signal[n];
	 }
	 
	 return max_val;
}

float min_vrednost (int dolzina_tabele, float *signal)
{
	int k = 0;
	float min_val = signal[0];
	
	for(k=0;k<dolzina_tabele;k++)
	 {
		if (signal[k]<min_val) min_val=signal[k];
	 }
	 
	 return min_val;
}

void reset_tocke ()
{
	tockax1 = 0;
	tockay1 = 140;
	tockax2 = 0;
	tockay2 = 0;
}

void ADC_Init()
{
	// desna poravnava
	ADMUX |= (1<<REFS0);
	ADCSRA |= (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

uint16_t ADC_read()
{
	ADCSRA |= (1<<ADSC); // start ADC pretvorbe
	while(ADCSRA & (1<<ADSC)){}; // pocaka da je ADSC = 0, konec pretvorbe
	return ADC;
}

void ADC_selectchannel(char channel)
{
	channel &= 0x7;
	ADMUX = (ADMUX & 0xF8)|channel;
}

float Izracun_Upp(int dolzina_tabele, float *signal)
{
	float Upp = 0;
	float max_abs = 0;
	float min_abs = 0;
	
	max_abs = max_vrednost (dolzina_tabele, signal);
	min_abs = min_vrednost (dolzina_tabele, signal);
	
	if(max_abs == min_abs) 
		{
			Upp =0;
			return Upp;
		}

	if (max_abs < 0) max_abs = -max_abs;   // absolutni vrednosti max. in min. clenov
	if (min_abs < 0) min_abs = -min_abs;
	
	Upp = max_abs - min_abs;
	
	return Upp;
}

void Refresh_display()
{
	UG_FillFrame(1, 39, 319, 238, C_BLACK);
	izris_ekrana();
}

int Izracun_frek (float *signal, float Udc, uint32_t cas_zajema)
{
	float tmp = 0;
	int cl = 0;
	
	for(cl=1;cl<320;cl++)
	{
		if((signal[cl-1]<Udc) && (signal[cl+1]>Udc)) tmp++;
	}
	
	tmp = ((tmp/cas_zajema)*1000)/2;
	return tmp;
}

float Izracun_Udc(float *signal, int dolzina_tabele)
{
	int i_tmp = 0;
	float tmp = 0;
	for (i_tmp=0;i_tmp<dolzina_tabele;i_tmp++) tmp = tmp + signal[i_tmp];
	tmp = tmp/dolzina_tabele;
	return tmp;
}